@@ -21,8 +21,6 @@
 #define APR_WANT_STRFUNC
 #include "apr_want.h"
 
-#define WANT_BASENAME_MATCH
-
 #include "httpd.h"
 #include "http_core.h"
 #include "http_config.h"
@@ -31,9 +29,6 @@
 #include "http_log.h"
 
 #include <iconv.h>
-#ifndef WIN32
-#include <pwd.h>
-#endif
 
 #ifndef _UNISTD_H
 #ifndef WIN32
@@ -55,6 +50,14 @@
 #define REDURL_ICONV_TRUE 0
 #define REDURL_ICONV_FALSE 1
 
+#if MODULE_MAGIC_NUMBER_MAJOR <= 20051115 // until httpd 2.2
+#	define REDURL_LOG_MARK APLOG_MARK
+#	define MODURL_LOG_MARK file,line
+#else                                     // from httpd 2.4
+#	define REDURL_LOG_MARK __FILE__,__LINE__
+#	define MODURL_LOG_MARK file,line,APLOG_MODULE_INDEX
+#endif
+
 /*
  * mod_url.c: fix mismatched URL encoding between server and clients
  * Writer:
@@ -63,7 +66,7 @@
  * URL:
  *   http://modurl.kldp.net/
  *
- * $Id: mod_url.c,v 1.16 2012/01/24 17:19:11 oops Exp $
+ * $Id: mod_url.c,v 1.21 2014/03/24 16:22:57 oops Exp $
  */
 
 /*
@@ -235,10 +238,10 @@ char * check_redurl_encode (const char * str, int len, int * retlen)
 	return (char *) r;
 }
 
-void redurl_mem_error (const request_rec * s, const char * file, int line, char * r)
+void redurl_mem_error (const request_rec * r, const char * file, int line, char * msg)
 {
-	ap_log_rerror (file, line, APLOG_ERR, APR_ENOPOOL, s,
-			"%s variable: memory allocation failed", r);
+	ap_log_rerror (MODURL_LOG_MARK, APLOG_ERR, APR_ENOPOOL, r,
+			"%s variable: memory allocation failed", msg);
 }
 
 void check_redurl_iconv (request_rec * r, urlconfig * cfg, iconv_s * ic, char * s_uri)
@@ -288,7 +291,7 @@ void check_redurl_iconv (request_rec * r, urlconfig * cfg, iconv_s * ic, char *
 
 	if ( (ic->uri = (char *) malloc (sizeof (char) * tlen)) == NULL ) {
 		ic->ret = -1;
-		redurl_mem_error (r, APLOG_MARK, "ic->uri");
+		redurl_mem_error (r, REDURL_LOG_MARK, "ic->uri");
 		ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
 				"check_redurl_iconv: iconv convert end   -------------------");
 		iconv_close (cfg->cd);
@@ -308,9 +311,21 @@ void check_redurl_iconv (request_rec * r, urlconfig * cfg, iconv_s * ic, char *
 	iconv_close (cfg->cd);
 
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  Oirg       => %s (%d)", s_uri, ic->len);
+#if defined(__x86_64__) || defined(__ppc64__)
+			"  Oirg       => %s (%lu)",
+#else
+			"  Oirg       => %s (%u)",
+#endif
+			s_uri, ic->len
+	);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  Fixed      => %s (%d)", ic->uri, tlen);
+#if defined(__x86_64__) || defined(__ppc64__)
+			"  Fixed      => %s (%lu)",
+#else
+			"  Fixed      => %s (%u)",
+#endif
+			ic->uri, tlen
+	);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
 			"  Check Code => %d", ic->ret);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
@@ -347,6 +362,11 @@ static int check_redurl (request_rec * r)
 	int				  flen, plen;
 	int				  uic_r, ric_r;
 
+	ap_log_rerror(
+		APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
+		"** [check_redurl] Fixup hooking"
+	);
+
 	cfg = ap_get_module_config (r->per_dir_config, &redurl_module);
 	if ( ! cfg->enabled )
 		return DECLINED;
@@ -380,7 +400,7 @@ static int check_redurl (request_rec * r)
 	plen = r->path_info ? strlen (r->path_info) : 0;
 
 	if ( (realpath = (char *) malloc (sizeof (char) * (flen + plen + 1))) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "realpath");
+		redurl_mem_error (r, REDURL_LOG_MARK, "realpath");
 		return DECLINED;
 	}
 
@@ -407,7 +427,7 @@ static int check_redurl (request_rec * r)
 			"URI Converting");
 
 	if ( (uic = (iconv_s *) malloc (sizeof (iconv_s) + 1)) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "uic");
+		redurl_mem_error (r, REDURL_LOG_MARK, "uic");
 		free (realpath);
 
 		return DECLINED;
@@ -422,11 +442,25 @@ static int check_redurl (request_rec * r)
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
 			"  URI   => %s", uic->uri);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  S_LEN => %d", uic->len);
+#if defined(__x86_64__) || defined(__ppc64__)
+			"  S_LEN => %lu",
+#else
+			"  S_LEN => %u",
+#endif
+			uic->len
+	);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  LEN   => %d", uic->tlen);
+#if defined(__x86_64__) || defined(__ppc64__)
+			"  LEN   => %lu",
+#else
+			"  LEN   => %u",
+#endif
+			uic->tlen
+	);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  CODE  => %d", uic->ret);
+			"  CODE  => %d",
+			uic->ret
+	);
 
 	if ( uic_r == REDURL_ICONV_FALSE ) {
 		/* converting failed */
@@ -444,7 +478,7 @@ static int check_redurl (request_rec * r)
 	}
 
 	if ( (enc = check_redurl_encode (uic->uri, strlen (uic->uri), NULL)) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "enc");
+		redurl_mem_error (r, REDURL_LOG_MARK, "enc");
 		check_redurl_iconv_free (uic);
 		free (realpath);
 
@@ -463,7 +497,7 @@ static int check_redurl (request_rec * r)
 			"Real path Converting"); 
 
 	if ( (ric = (iconv_s *) malloc (sizeof (iconv_s) + 1)) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "ric");
+		redurl_mem_error (r, REDURL_LOG_MARK, "ric");
 		free (realpath);
 
 		return DECLINED;
@@ -479,9 +513,21 @@ static int check_redurl (request_rec * r)
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
 			"  PATH   => %s", ric->uri);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  S_LEN  => %d", ric->len);
+#if defined(__x86_64__) || defined(__ppc64__)
+			"  S_LEN  => %lu",
+#else
+			"  S_LEN  => %u",
+#endif
+			ric->len
+	);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
-			"  LEN    => %d", ric->tlen);
+#if defined(__x86_64__) || defined(__ppc64__)
+			"  LEN    => %lu",
+#else
+			"  LEN    => %r",
+#endif
+			ric->tlen
+	);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
 			"  CODE   => %d", ric->ret);
 
@@ -568,6 +614,11 @@ static int pre_redurl (request_rec * r) {
 	struct		  stat realstat;
 	short		  is_userdir = 0;
 
+	ap_log_rerror(
+		APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r,
+		"** [pre_redurl] Translate name hooking"
+	);
+
 	cfg = ap_get_module_config (r->per_dir_config, &redurl_module);
 	if ( ! cfg->enabled )
 		return DECLINED;
@@ -587,124 +638,7 @@ static int pre_redurl (request_rec * r) {
 
 #ifndef WINNT
 
-	/* Account access : /~user */
-	if ( r->uri[0] == '/' && r->uri[1] == '~' ) {
-#ifndef URL_NOUSERDIR
-#if defined(WIN32) || defined(NETWARE)
-		/* Need to figure out home dirs on NT and NetWare 
-		 * Shouldn't happen here, though, we trap for this in set_user_dir
-		 */
-		return DECLINED;
-#else
-		extern module AP_MODULE_DECLARE_DATA userdir_module;
-		typedef struct userdir_config {
-			int globally_disabled;
-			char *userdir;
-			apr_table_t *enabled_users;
-			apr_table_t *disabled_users;
-		} userdir_config;
-
-		const userdir_config *ucfg;
-		const char *userdirs;
-		struct passwd *pw;
-		const char *w, *dname;
-
-		is_userdir = 1;
-
-		ucfg = (userdir_config *)
-				ap_get_module_config (r->server->module_config, &userdir_module);
-		userdirs = ucfg->userdir;
-
-		if ( ucfg->userdir == NULL )
-			return DECLINED;
-
-		dname = r->uri + 2;
-		w = ap_getword (r->pool, &dname, '/');
-
-		/*
-		 * The 'dname' funny business involves backing it up to capture the '/'
-		 * delimiting the "/~user" part from the rest of the URL, in case there
-		 * was one (the case where there wasn't being just "GET /~user HTTP/1.0",
-		 * for which we don't want to tack on a '/' onto the filename).
-		 */
-		if (dname[-1] == '/') {
-			--dname;
-		}
-		ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r, "UDir w => %s", w);
-		ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r, "UDir dname => %s", dname);
-
-		/*
-		 * If there's no username, it's not for us.  Ignore . and .. as well.
-		 */
-		if ((w[0] == '\0') || ((w[1] == '.') &&
-			((w[2] == '\0') || ((w[2] == '.') && (w[3] == '\0'))))) {
-			return DECLINED;
-		}
-
-		/*
-		 * Nor if there's an username but it's in the disabled list.
-		 */
-		if (apr_table_get(ucfg->disabled_users, w) != NULL)
-			return DECLINED;
-
-		/*
-		 * If there's a global interdiction on UserDirs, check to see if this
-		 * name is one of the Blessed.
-		 */
-		if (ucfg->globally_disabled
-				&& (apr_table_get(ucfg->enabled_users, w) == NULL))
-			return DECLINED;
-
-		while ( *userdirs) {
-			const char * userdir;
-			int is_absolute;
-
-			userdir = ap_getword_conf (r->pool, &userdirs);
-			is_absolute = ap_os_is_path_absolute (r->pool, userdir);
-
-			if ( strchr (userdir, '*') ) {
-				char *x = ap_getword (r->pool, &userdir, '*');
-
-				if ( is_absolute ) {
-					/* token '*' within absolute path
-					 * serves [UserDir arg-pre*][user][UserDir arg-post*]
-					 * /somepath/ * /somedir + /~smith -> /somepath/smith/somedir
-					 */
-					sprintf (docroot, "%s%s%s%s", x, w, userdir, dname);
-				} else {
-					return DECLINED;
-				}
-			} else if ( is_absolute ) {
-				/* An absolute path, no * token:
-				 * serves [UserDir arg]/[user]
-				 * /home + /~smith -> /home/smith
-				 */
-				if ( userdir[strlen (userdir) - 1] == '/' )
-					sprintf (docroot, "%s%s%s", userdir, w, dname);
-				else
-					sprintf (docroot, "%s/%s%s", userdir, w, dname);
-			} else if ( strchr (userdir, ':') ) {
-				return DECLINED;
-			} else {
-				struct passwd *pw;
-
-				if ( (pw = getpwnam (w)) ) {
-#ifdef OS2
-					/* Need to manually add user anem for OS/2 */
-					sprintf (docroot, "%s%s/%s%s", pw->pw_dir, w, userdir, dname);
-#else
-					sprintf (docroot, "%s/%s%s", pw->pw_dir, userdir, dname);
-#endif
-				}
-			}
-		}
-#endif
-#else
-		return DECLINED;
-#endif /* URL_NOUSERDIR */
-	} else {
-		strcpy (docroot, ap_document_root (r));
-	}
+	strcpy (docroot, ap_document_root (r));
 	rootlen = strlen (docroot);
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r, "UDir docroot => %s", docroot);
 
@@ -718,7 +652,7 @@ static int pre_redurl (request_rec * r) {
 	real_len = rootlen + urilen;
 
 	if ( (realpath = (char *) malloc (sizeof (char) * (real_len + 1))) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "pre realpath");
+		redurl_mem_error (r, REDURL_LOG_MARK, "pre realpath");
 		return DECLINED;
 	}
 
@@ -747,7 +681,7 @@ static int pre_redurl (request_rec * r) {
 			"Pre Real path Converting"); 
 
 	if ( (ric = (iconv_s *) malloc (sizeof (iconv_s) + 1)) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "pre real: ric");
+		redurl_mem_error (r, REDURL_LOG_MARK, "pre real: ric");
 		free (realpath);
 
 		return DECLINED;
@@ -785,7 +719,7 @@ static int pre_redurl (request_rec * r) {
 	ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, r, "Pre URI Converting");
 
 	if ( (ric = (iconv_s *) malloc (sizeof (iconv_s) + 1)) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "pre uri: ric");
+		redurl_mem_error (r, REDURL_LOG_MARK, "pre uri: ric");
 		free (realpath);
 		return DECLINED;
 	}
@@ -800,7 +734,7 @@ static int pre_redurl (request_rec * r) {
 	}
 
 	if ( (enc = check_redurl_encode (ric->uri, strlen (ric->uri), NULL)) == NULL ) {
-		redurl_mem_error (r, APLOG_MARK, "enc");
+		redurl_mem_error (r, REDURL_LOG_MARK, "enc");
 		check_redurl_iconv_free (ric);
 		free (realpath);
 
